<?php

namespace Drupal\challenge_mitigation;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * List builder for CM whitelist IP entities.
 *
 * Displays IP addresses that have been temporarily whitelisted
 * after passing the challenge. Includes metadata such as origin
 * and creation date, along with basic operations.
 */
class ChallengeMitigationWhitelistIpListBuilder extends EntityListBuilder {

  /**
   * Constructs a new list builder.
   */
  public function __construct(
    $entityType,
    $storage,
    /**
     * The date formatter service.
     */
    protected DateFormatterInterface $dateFormatter,
  ) {
    parent::__construct($entityType, $storage);
  }

  /**
   * {@inheritdoc}
   *
   * Injects services from the container.
   */
  public static function createInstance(ContainerInterface $container, $entityType): static {
    return new static(
      $entityType,
      $container->get('entity_type.manager')->getStorage($entityType->id()),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   *
   * Defines the table headers used in the admin list.
   */
  public function buildHeader(): array {
    $header['ip'] = $this->t('IP address');
    $header['origin'] = $this->t('Origin');
    $header['created'] = $this->t('Created');
    $header['operations'] = $this->t('Operations');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   *
   * Defines how each row of the list is rendered.
   */
  public function buildRow(EntityInterface $entity): array {
    $row['ip'] = $entity->get('ip')->value;
    $row['origin'] = $entity->get('origin')->value;
    $row['created'] = $this->dateFormatter->format($entity->get('created')->value);
    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   *
   * Enables pagination by limiting the number of items loaded.
   */
  public function load(): array {
    $query = $this->getStorage()->getQuery()
      ->accessCheck(TRUE)
      ->pager(50);

    $ids = $query->execute();
    return $this->storage->loadMultiple($ids);
  }

}
